/**
 * JHOVE2 - Next-generation architecture for format-aware characterization
 *
 * Copyright (c) 2009 by The Regents of the University of California,
 * Ithaka Harbors, Inc., and The Board of Trustees of the Leland Stanford
 * Junior University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * o Neither the name of the University of California/California Digital
 *   Library, Ithaka Harbors/Portico, or Stanford University, nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.jhove2.module.format.arc;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.text.DecimalFormat;
import java.util.List;

import javax.annotation.Resource;

import org.jhove2.app.util.FeatureConfigurationUtil;
import org.jhove2.config.spring.SpringConfigInfo;
import org.jhove2.core.JHOVE2;
import org.jhove2.core.JHOVE2Exception;
import org.jhove2.core.io.Input;
import org.jhove2.core.source.FileSource;
import org.jhove2.core.source.Source;
import org.jhove2.module.display.Displayer;
import org.jhove2.module.display.XMLDisplayer;
import org.jhove2.module.format.BaseFormatModule;
import org.jhove2.module.format.gzip.GzipModule;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;


@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = {
		"classpath*:**/arc-test-config.xml",
        "classpath*:**/jhove2-gzip-config.xml", // Standard GZip module config.
        "classpath*:**/test-config.xml",
        "classpath*:**/filepaths-config.xml" })
public class ArcModuleTest {

	private String arcDirBasePath;
	private ArcModule arcModule;
	private GzipModule gzipModule;
	private JHOVE2 JHOVE2;
    
	@Resource(name = "ArcModule")
	public void setArcModule(ArcModule ArcModule) {
		this.arcModule = ArcModule;
	}

	@Resource(name = "TestGzipModule")
	public void setGzipModule(GzipModule module) {
		this.gzipModule = module;
	}

	@Resource
	public void setJHOVE2(JHOVE2 JHOVE2) {
    	this.JHOVE2 = JHOVE2;
	 }

	@Test
	@Ignore
    public void parseArcFile() {
		/*
		String samplesDirPath = null;
		try {
			samplesDirPath = FeatureConfigurationUtil.getFilePathFromClasspath(arcDirBasePath, "file samples dir");
		} catch (JHOVE2Exception e1) {
			fail("Could not create base directory");
		}
		String relativePath = "small_BNF.arc";
        File arcFile = new File(samplesDirPath,relativePath);
        assertTrue(arcFile.exists());
        try {
            FileSource source = new FileSource(arcFile);
            Input input = source.getInput(JHOVE2);
            final DecimalFormat format = new DecimalFormat();
            format.setMinimumFractionDigits(3);
            format.setMaximumFractionDigits(3);
            long start = System.nanoTime();
         
            arcModule.parse(JHOVE2, source, input);
            double characterizationDuration = 
            				(System.nanoTime() - start) / 1000000000.0; // sec
            System.out.println("Characterization duration : " + format.format(characterizationDuration));
            Displayer displayer = SpringConfigInfo.getReportable(Displayer.class,
					                           Displayer.DEFAULT_DISPLAYER_TYPE);
			displayer.setConfigInfo(JHOVE2.getConfigInfo());
			displayer.display(source);
            assertEquals(6, source.getNumChildSources());
            int i = 0;
            for(Source s : source.getChildSources()){
            	Class<?> clazz = (i++ == 0)? VersionBlockSource.class 
		                   : ArcRecordSource.class;
            	assertTrue(clazz.isAssignableFrom(s.getClass()));
            	assertFalse(((ArcRecordBaseSource<?>)s).isNonCompliant());
            }
            List<Source> sources = source.getChildSources();
            VersionBlockSource versionBlock = (VersionBlockSource)sources.get(0);
            String path = "BNF-FS-072076.arc.gz";
            String ipAddress = "0.0.0.0";
            String rawDate = "20060305082251";
            String contentType = "text/plain";
            Long archiveLength = new Long(76L);
            assertEquals(path,versionBlock.getSourceName());
            assertEquals(ipAddress,versionBlock.getIpAddress());
            assertEquals(rawDate,versionBlock.getRawArchiveDate());
            assertEquals(contentType,versionBlock.getContentType());
            assertEquals(archiveLength,versionBlock.getLength());
            if(arcModule.getParallelCharacterization() == 1){
            	ArcRecordSource arcRecord = (ArcRecordSource)sources.get(3);
            	path = "http://chss.montclair.edu:80/psychology/perera/0012c.gif";
            	ipAddress = "130.68.1.43";
            	rawDate = "19970417175712";
            	contentType = "image/gif";
            	archiveLength = new Long(26982);
            	assertEquals(path,arcRecord.getSourceName());
            	assertEquals(ipAddress,arcRecord.getIpAddress());
            	assertEquals(rawDate,arcRecord.getRawArchiveDate());
            	assertEquals(contentType,arcRecord.getContentType());
            	assertEquals(archiveLength,arcRecord.getLength());
            	assertEquals("image/gif",arcRecord.getProtocolContentType());
            	assertEquals(new Integer(200),arcRecord.getProtocolResultCode());
            	assertEquals("HTTP/1.0",arcRecord.getProtocolVersion());
            }
        }
        catch (Exception e) {
        	e.printStackTrace();
            fail("Exception thrown: " + e.getMessage());
        }
        */
    }

    @Test
    @Ignore
	public void parseArcFiles() {
    	/*
//	String samplesDirPath = null;
//	try {
//	    samplesDirPath = FeatureConfigurationUtil.getFilePathFromClasspath(arcDirBasePath, "file samples dir");
//	} catch (JHOVE2Exception e1) {
//	    fail("Could not create base directory");
//	}
//	String relativePath = "tocharacterize";
//      File arcDitrectory = new File(samplesDirPath,relativePath);
//      assertTrue(arcDitrectory.exists() && arcDitrectory.isDirectory());
        // Create output formatter.
        final DecimalFormat format = new DecimalFormat();
        format.setMinimumFractionDigits(3);
        format.setMaximumFractionDigits(3);
        
        try {
			System.setOut(new PrintStream(new FileOutputStream("/home/hbian/SVN_JHOVE2/tmp/ArcModuleTestResult_arcgz_jna_4threads_20110103.csv")));
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
//		System.out.println("Default tmp directory : " + System.getProperty("java.io.tmpdir"));
		System.setProperty("java.io.tmpdir", "/tmp/hui");
//		System.out.println("tmp directory to use : " + System.getProperty("java.io.tmpdir"));
		
		for(int i = 0; i<1; i++) {
			i = 0;
//			File toProcess = (i == 0) ? new File(arcDitrectory, "ARCgz") : new File(arcDitrectory, "ARC");
			File toProcess = new File("/home/hbian/SVN_JHOVE2/target/test-classes/examples/arc/tocharacterize/ARC");
			assertTrue(toProcess.exists());
			BaseFormatModule module = (i == 0) ? gzipModule : arcModule;
//			File toProcess = new File(arcDitrectory, "ARCgz");
			
//			String[] fileArcTab = new String[]{
//					"557-2-20100627190528-01458-BnF_large_2010_01_atlas307.bnf.fr.arc",
//					"BNF-CRAWL-000-20041222160420-00490-crawling009.archive.org.arc",
//					"BNF-CRAWL-001-20051013135526-02420-crawling017.arc",
//					"BNF-CRAWL-002-20061128181000-07504-crawling015.us.archive.org.arc",
//					"BNF-DW_crawl25.20040528185046.arc",
//					"BnF-cibl2008-20081130031432-00482-atlas20.arc",
//					"BnF-cibl2009-20090307205247-00334-atlas10.bnf.fr.arc",
//					"small_BNF.arc"
//			};
			
			String[] fileArcGzTab = new String[]{
					"434-1-20100428193450-00039-BnF_large_2010_01_atlas607.bnf.fr.arc.gz",
					"BNF-FOCUSEDCRAWL-002-20061125193047-09477-crawling03.us.archive.org.arc.gz",
					"BNF-short1-2-25-957487343.arc.gz",
					"BnF-Dailymotion-20091205023333-00262-atlas10.bnf.fr.arc.gz",
					"BNF-FRAGMENT-ia108643.20050307093826.arc.gz",
					"BnF-Focused2007-20071128101709-00426-atlas1.arc.gz",
					"557-2-20100627190528-01458-BnF_large_2010_01_atlas307.bnf.fr.arc.gz",
					"BnF-elec2010-20100420204356-00035-atlas10.bnf.fr.arc.gz",
					"BNF-FS-036938.arc.gz",
					"BnF-elec2002-lingway.20080123054906.arc.gz",
					"BnF-cibl2009-20090307205247-00334-atlas10.bnf.fr.arc.gz",
					"BNF-EC_crawl22.20050307144930.arc.gz",
					"BnF-elec2007-20061019234315-00007-atlas3.arc.gz",
					"BNF-test4-0-256-956942633.arc.gz",
					"BNF-CRAWL-004-20081116155718-06859-crawling107.us.archive.org.arc.gz",
					"BNF-FS-079206.arc.gz",
					"BNF-green-000325-19990428131510-925352666.arc.gz",
					"BNF-crc14-2-aa-958469085.arc.gz",
					"BNF-DW_crawl25.20040528185046.arc.gz",
					"BNF-ST-000236.arc.gz",
					"BnF-elec2007-20070122113132-00010-atlas1.arc.gz",
					"BNF-DE_crawl2.20010525231014.arc.gz",
					"BNF-to-crawl-000003-20000519142052-958802161.arc.gz",
					"BNF-CRAWL-001-20051013135526-02420-crawling017.arc.gz",
					"BNF-DQ_slash_crawl8.20030509231229.arc.gz",
					"BNF-CRAWL-000-20041222160420-00490-crawling009.archive.org.arc.gz",
					"BNF-inktomi_arc39.20011005194912.arc.gz",
					"BNF-sarah-000361-20000509173617-957964125.arc.gz",
					"BNF-EC_binary1_crawl30.20050304232237.arc.gz",
					"BNF-CRAWL-002-20061128181000-07504-crawling015.us.archive.org.arc.gz",
					"BNF-GR-473716.arc.gz",
					"BNF-FOCUSEDCRAWL-001-20051016155943-01565-crawling014.arc.gz",
					"BNF-DK_crawl6.20020430150410.arc.gz",
					"BNF-ED_slash_crawl32.20050330204646.arc.gz",
					"166-1-20100419224155-00242-BnF_large_2010_01_atlas603.bnf.fr.arc.gz",
					"IAH-20060928183427-00084-atlas3.arc.gz",
					"BNF-CRAWL-003-20071019120617-08127-crawling06.us.archive.org.arc.gz",
					"BNF-CRAWL-002-20061128203314-08006-crawling04.us.archive.org.arc.gz"
			};
			
			StringBuilder sb = null;
			File f = null;
//	        for(File f : toProcess.listFiles()){
			for(String filePath : fileArcGzTab){
				f = new File("/home/hbian/SVN_JHOVE2/target/test-classes/examples/arc/tocharacterize/ARCgz/" + filePath);
	        	sb = new StringBuilder();
//	        File f = new File("/home/hbian/SVN_JHOVE2/target/test-classes/examples/arc/tocharacterize/ARCgz/BnF-Focused2007-20071128101709-00426-atlas1.arc.gz");
	  	    
//	        for (int i = 1; i < 6; i++) {
			
	        try {
			  	    System.setErr(new PrintStream(new FileOutputStream("/home/hbian/SVN_JHOVE2/tmp/err/" + f.getName() + "_err.txt")));
//			  		System.out.println("-------------------------------------");
//			  		System.out.println("Processing the file : " + f.getName());
			  	    sb.append("03/01/2011;HBN; ;").append(f.getName()).append(";ARC.GZ;");
			  		
			  	    FileSource source = new FileSource(f);
	                Input input = source.getInput(JHOVE2);
	                module.parse(JHOVE2, source, input);
//	                System.out.println("Source size : " + 
//	  				String.valueOf(source.getSize()));
	                
	                sb.append(String.valueOf(source.getSize())).append(";");
	                
	                int arcRecordNumber = (source.getNumChildSources() > 1) 
	                                       ? source.getNumChildSources() -1 : 0;
//	                System.out.println("Analysed arcRecords number : " + 
//	  				String.valueOf(arcRecordNumber));
	                
	                sb.append(String.valueOf(arcRecordNumber)).append(";aristote3;Y;4;Jna;Y;");
	                
//	                System.out.println("Elapsed time : " + 
//	  				source.getTimerInfo().getElapsedTime());
	                
	                sb.append(String.valueOf(source.getTimerInfo().getElapsedTime().getDuration() / 1000.0).replace('.', ',')).append(";");
	                
	                long start = System.nanoTime();
	                
//	                Displayer displayer = SpringConfigInfo.getReportable(Displayer.class,
//	    					Displayer.DEFAULT_DISPLAYER_TYPE);
//	    			displayer.setConfigInfo(JHOVE2.getConfigInfo());
//	    			String path = "/home/hbian/SVN_JHOVE2/tmp/txt/" + f.getName() + ".txt";
//	          		System.out.println("txt path : " + path);
//	          		displayer.display(source, path);
	                
	                Displayer displayer = new XMLDisplayer();
	          		displayer.setConfigInfo(JHOVE2.getConfigInfo());
	          		String path = "/tmp/hui/" + f.getName() + ".xml";
//	          		System.out.println("xml path : " + path);
	          		displayer.display(source, path);
	          		
	          		double xmlGenerationTime = (System.nanoTime() - start) / 1000000000.0; // sec
	          	    double characterizationTime = source.getTimerInfo().getElapsedTime().getDuration() / 1000.0; //sec
//	          	    System.out.println("Result file generation time : "  +  String.valueOf(xmlGenerationTime) + " in seconds");
	          	    
	          	    sb.append(String.valueOf(xmlGenerationTime).replace('.', ',')).append(";");
	          	    
//	          	    System.out.println("Execution time  : " + String.valueOf(characterizationTime + xmlGenerationTime) + " in seconds");
	          	    
	          	    sb.append(String.valueOf(characterizationTime + xmlGenerationTime).replace('.', ',')).append(";");
	          	    sb.append(String.valueOf(new File(path).length())).append(";Y;xml");
	          	    System.out.println(sb.toString());
	          	    //System.out.println("Execution time : " + format.format(characterizationTime + xmlGenerationTime));
	                  //Displayer displayer = SpringConfigInfo.getReportable(Displayer.class,
	  		//						     Displayer.DEFAULT_DISPLAYER_TYPE);
	  		//displayer.setConfigInfo(JHOVE2.getConfigInfo());
	  		//displayer.display(source);   
	  	    }
	  	    catch (Exception e) {
	  	    	e.printStackTrace();
	  		//fail("Exception thrown: " + e.getMessage());
	  	    } finally {
	  	    	//System.out.println("-------------------------------------");
	  	    }
	       }
		}
		*/
    }

	@Test    
	@Ignore
    public void parse() {
    	/*
		String samplesDirPath = null;
		try {
			samplesDirPath = FeatureConfigurationUtil.getFilePathFromClasspath(arcDirBasePath, "file samples dir");
		} catch (JHOVE2Exception e1) {
			fail("Could not create base directory");
		}
		String relativePath = "toto.gz";
        File arcFile = new File(samplesDirPath,relativePath);
        assertTrue(arcFile.exists());
        try {
            FileSource source = new FileSource(arcFile);
            Input input = source.getInput(JHOVE2);
            //arcModule.parse(JHOVE2, source, input);
            gzipModule.parse(JHOVE2, source, input);
            Displayer displayer = SpringConfigInfo.getReportable(Displayer.class,
					Displayer.DEFAULT_DISPLAYER_TYPE);
			displayer.setConfigInfo(JHOVE2.getConfigInfo());
			displayer.display(source);	
        }
        catch (Exception e) {
        	e.printStackTrace();
            fail("Exception thrown: " + e.getMessage());
        }
        */
    }

    /**
	 * fileDirBasePath setter
	 * @param fileDirBasePath the fileDirBasePath to set
	 */
	@Resource
	public void setArcDirBasePath(String path) {
		this.arcDirBasePath = path;
	}

}
