package org.jhove2.module.format.warc;

import static org.junit.Assert.fail;

import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import javax.annotation.Resource;

import org.jhove2.core.JHOVE2Exception;
import org.jhove2.core.io.Input;
import org.jhove2.core.source.FileSource;
import org.jhove2.core.source.Source;
import org.jhove2.module.display.Displayer;
import org.jhove2.module.display.TextDisplayer;
import org.jhove2.module.format.gzip.GzipModule;
import org.junit.Test;
import org.springframework.test.context.ContextConfiguration;

@ContextConfiguration(locations = {
		"classpath*:**/warc-test-config.xml",
		"classpath*:**/gzip-test-config.xml" })
public class WarcModuleBaseTest extends ModuleTestBase {

	protected WarcModule warcModule;
	protected GzipModule gzipModule;
	protected String warcDirBasePath;
    protected boolean initialized;

    public WarcModule getWarcModule() {
    	return warcModule;
    }

    @Resource(name = "WarcModule")
    public void setWarcModule(WarcModule warcModule) {
    	this.warcModule = warcModule;
    }

    public GzipModule getGzipModule() {
    	return gzipModule;
    }

    @Resource(name = "GZipModule")
    public void setGzipModule(GzipModule gzipModule) {
    	this.gzipModule = gzipModule;
    }

    public String getWarcDirBasePath() {
        return warcDirBasePath;
    }

    @Resource
    public void setWarcDirBasePath(String warcDirBasePath) {
        this.warcDirBasePath = warcDirBasePath;
    }

    protected void parse(String relativePathStr) {
    	fileSource = getTestFileSource(warcDirBasePath, relativePathStr, "WARC examples base directory");
        try {
            input = fileSource.getInput(JHOVE2);
            warcModule.parse(JHOVE2, fileSource, input);
        }
        catch (Exception e) {
            fail("Exception thrown: " + e.getMessage());
        }
    }

    public void display() {
        try {
            input.setPosition(0);
            JHOVE2.characterize(fileSource, input);
            Displayer displayer = new TextDisplayer();
            displayer.setConfigInfo(JHOVE2.getConfigInfo());
            displayer.display(fileSource);          
        }
        catch (Exception e){
            e.printStackTrace();
            fail("exception");
        }
    }

    protected FileSource fileSource;
    protected Input input;

    @Test
    public void testWarcFieldsWarcinfo() {
        try {
        	parse( "test-fields-warcinfo.warc" );

        	fileSource.addModule( warcModule );
        	//warcModule.setParentSourceId( fileSource.getSourceId() );
        	//warcModule = (WarcModule)warcModule.getModuleAccessor().persistModule( warcModule );

        	System.out.println( warcModule.isValid() );
			List<Source> children = fileSource.getChildSources();

	        Displayer displayer = new TextDisplayer();
	        displayer.setConfigInfo(JHOVE2.getConfigInfo());
			displayer.display(fileSource);
		}
        catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        catch (UnsupportedEncodingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        catch (JHOVE2Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}          
    }

    @Test
    public void testWarcFieldsMetainfo() {
        try {
        	parse( "test-fields-metainfo.warc" );

        	fileSource.addModule( warcModule );
        	//warcModule.setParentSourceId( fileSource.getSourceId() );
        	//warcModule = (WarcModule)warcModule.getModuleAccessor().persistModule( warcModule );

        	System.out.println( warcModule.isValid() );
			List<Source> children = fileSource.getChildSources();

	        Displayer displayer = new TextDisplayer();
	        displayer.setConfigInfo(JHOVE2.getConfigInfo());
			displayer.display(fileSource);
		}
        catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        catch (UnsupportedEncodingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        catch (JHOVE2Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}          
    }

    @Test
    public void testWarcFieldsContinuation() {
        try {
        	parse( "test-fields-continuation.warc" );

        	fileSource.addModule( warcModule );
        	//warcModule.setParentSourceId( fileSource.getSourceId() );
        	//warcModule = (WarcModule)warcModule.getModuleAccessor().persistModule( warcModule );

        	System.out.println( warcModule.isValid() );
			List<Source> children = fileSource.getChildSources();

	        Displayer displayer = new TextDisplayer();
	        displayer.setConfigInfo(JHOVE2.getConfigInfo());
			displayer.display(fileSource);
		}
        catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        catch (UnsupportedEncodingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        catch (JHOVE2Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}          
    }

}
